/*!
 * @vueblocks/global v1.0.0-alpha.2
 * (c) 2021 xiaoluoboding
 * @license MIT
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi'), require('echarts/core'), require('@vueblocks/vue-use-core'), require('echarts/charts'), require('lodash.clonedeep'), require('echarts/components'), require('echarts/renderers')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi', 'echarts/core', '@vueblocks/vue-use-core', 'echarts/charts', 'lodash.clonedeep', 'echarts/components', 'echarts/renderers'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VeCharts = {}, global.VueDemi, global.EchartsCore, global.VueUseCore, global.EchartsCharts, global.CloneDeep, global.EchartsComponents, global.EchartsRenderers));
}(this, (function (exports, vueDemi, core, vueUseCore, charts, cloneDeep, components, renderers) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var cloneDeep__default = /*#__PURE__*/_interopDefaultLegacy(cloneDeep);

  const toKebabCase = (str) => str
      .replace(/([A-Z])([A-Z])/g, '$1-$2')
      .replace(/([a-z])([A-Z])/g, '$1-$2')
      .replace(/[\s_]+/g, '-')
      .toLowerCase();
  const toCamelCase = (arr) => arr
      .map((str, idx) => idx > 0 ? str.charAt(0).toLocaleUpperCase() + str.slice(1) : str)
      .join('');
  const isEmpty = (prop) => prop === null || prop === undefined;
  // Copied from
  // https://github.com/vuejs/vue-next/blob/5a7a1b8293822219283d6e267496bec02234b0bc/packages/shared/src/index.ts#L40-L41
  const isOn = (key) => /^on[^a-z]/.test(key);
  function omitOn(attrs) {
      const result = {};
      for (const key in attrs) {
          if (!isOn(key)) {
              result[key] = attrs[key];
          }
      }
      return result;
  }
  const withInstall = (comp) => {
      const c = comp;
      c.install = (app) => {
          app.component(c.name, c);
      };
      return c;
  };

  /* eslint-disable @typescript-eslint/no-explicit-any */
  const useAttrs = (attrs) => {
      const echartsAttrs = vueDemi.computed(() => omitOn(attrs));
      const echartsEvents = {};
      Object.keys(attrs)
          .filter(key => isOn(key) && typeof attrs[key] === 'function')
          .forEach(key => {
          const [, ...rest] = toKebabCase(key).split('-');
          const eventKey = rest.length === 1
              ? rest.join('')
              : toCamelCase(rest);
          echartsEvents[eventKey] = attrs[key];
      });
      return {
          echartsAttrs,
          echartsEvents
      };
  };

  /* eslint-disable @typescript-eslint/no-explicit-any */
  var Chart$9 = vueDemi.defineComponent({
      name: 'VeChart',
      inheritAttrs: false,
      props: {
          option: Object,
          initOptions: Object,
          theme: [String, Object],
          setOptionOpts: Object,
          height: { type: Number, default: 400 },
          needUpdate: { type: Boolean, default: false }
      },
      setup(props, { attrs }) {
          const option = vueDemi.toRef(props, 'option');
          const theme = vueDemi.toRef(props, 'theme');
          const initOptions = vueDemi.toRef(props, 'initOptions');
          const setOptionOpts = vueDemi.toRef(props, 'setOptionOpts');
          const needUpdate = vueDemi.toRef(props, 'needUpdate');
          const echartsRef = vueDemi.ref();
          const echartsInstance = vueDemi.shallowRef();
          const canvasRect = vueDemi.ref({});
          const { echartsEvents } = useAttrs(attrs);
          console.log(option.value);
          const resize = () => {
              if (echartsInstance.value) {
                  echartsInstance.value.resize();
              }
          };
          const dispose = () => {
              if (echartsInstance.value) {
                  echartsInstance.value.dispose();
                  echartsInstance.value = undefined;
              }
          };
          const delegateEvents = (instance) => {
              Object.keys(echartsEvents).forEach(key => {
                  const handler = echartsEvents[key];
                  if (!handler)
                      return;
                  if (key.indexOf('zr:') === 0) {
                      instance.getZr().on(key.slice(3).toLowerCase(), handler);
                  }
                  else {
                      instance.on(key, handler);
                  }
              });
          };
          const init = (option) => {
              if (!echartsRef.value)
                  return;
              echartsInstance.value = core.init(echartsRef.value, props.theme, {
                  renderer: 'canvas'
              });
              console.log(echartsInstance.value);
              option && echartsInstance.value.setOption(option);
              delegateEvents(echartsInstance.value);
              vueDemi.nextTick(resize);
          };
          const setOption = (option, opts) => {
              if (!echartsInstance.value) {
                  init(option);
              }
              else {
                  echartsInstance.value.setOption(option, opts);
              }
          };
          vueUseCore.useResizeObserver(echartsRef, ([entry]) => {
              canvasRect.value = entry.contentRect;
              resize();
          });
          vueDemi.watch(needUpdate, (needUpdate) => {
              if (needUpdate && option.value) {
                  if (setOptionOpts.value) {
                      setOption(option.value, setOptionOpts.value);
                  }
                  else {
                      setOption(option.value);
                  }
              }
          }, { deep: true });
          vueDemi.watch([theme, initOptions], () => {
              dispose();
              init();
          }, { deep: true });
          vueDemi.onMounted(() => {
              option.value && init(option.value);
          });
          vueDemi.onUnmounted(dispose);
          return {
              echartsRef,
              echartsInstance,
              setOption
          };
      },
      render() {
          const vm = vueUseCore.useInstance();
          const echartsStyle = {
              width: 'auto',
              height: `${vm.$props.height}px`
          };
          return vueDemi.h('div', {
              ref: 'echartsRef',
              style: echartsStyle
          });
      }
  });

  const VeChart = withInstall(Chart$9);

  core.use([
      components.GridComponent,
      renderers.CanvasRenderer
  ]);

  /* eslint-disable @typescript-eslint/no-explicit-any */
  var HocChart = vueDemi.defineComponent({
      props: {
          /**
           * echarts options
           * ve-charts delegate all echarts options as component props
           * https://echarts.apache.org/zh/option.html
           */
          title: Object,
          legend: Object,
          grid: Object,
          xAxis: [Object, Array],
          yAxis: [Object, Array],
          polar: Object,
          radiusAxis: Object,
          angleAxis: Object,
          radar: [Object, Array],
          dataZoom: [Object, Array],
          visualMap: [Object, Array],
          tooltip: Object,
          axisPointer: Object,
          toolbox: Object,
          brush: Object,
          geo: Object,
          parallel: Object,
          parallelAxis: Array,
          singleAxis: Array,
          timeline: Object,
          graphic: Object,
          calendar: Object,
          dataset: Object,
          aria: Object,
          series: [Object, Array],
          darkMode: { type: [Boolean, String], default: 'auto' },
          color: Array,
          backgroundColor: [Object, String],
          textStyle: Object,
          animation: { type: Boolean, default: true },
          animationThreshold: Number,
          animationDuration: [Number, Function],
          animationEasing: [String, Function],
          animationDelay: [Number, Function],
          animationDurationUpdate: [Number, Function],
          animationEasingUpdate: [String, Function],
          animationDelayUpdate: [String, Function],
          stateAnimation: Object,
          blendMode: String,
          hoverLayerThreshold: Number,
          useUTC: { type: Boolean, default: false },
          options: Object,
          media: Array,
          // ve-charts basic props
          data: [Object, Array],
          settings: [Object, Array],
          loading: { type: Boolean, default: false },
          emptyText: String,
          // ve-charts common props
          option: {
              type: [Object, Array],
              default: () => { return {}; }
          }
      },
      data: () => ({
          initOptions: {
              renderer: 'canvas'
          },
          needUpdate: false,
          setOptionOpts: {},
          mergedOption: {}
      }),
      computed: {
          chartOpts() {
              return {
                  option: this.mergedOption,
                  initOptions: this.initOptions,
                  needUpdate: this.needUpdate,
                  setOptionOpts: this.setOptionOpts
              };
          }
      },
      created() {
          this.mergedOption = this.option;
          Object.keys(this.$props)
              // .forEach((prop: string) => (this.$props[prop] === undefined) && delete this.$props[prop])
              .filter((prop) => !/data|settings|height|loading|emptyText/.test(prop))
              .forEach((prop) => {
              this.$watch(prop, (val) => {
                  if (!isEmpty(val)) {
                      // merge echarts default option
                      const option = prop === 'option'
                          ? this.option
                          : { [prop]: val };
                      this.mergePropsToOption(option);
                  }
                  this.forcedUpdate();
              }, {
                  // TODO only object types need deep watch
                  deep: true,
                  immediate: true
              });
          });
      },
      beforeUnmount() {
          this.mergedOption = {};
      },
      methods: {
          mergePropsToOption(props) {
              const option = cloneDeep__default['default'](this.mergedOption);
              this.mergedOption = { ...option, ...props };
          },
          forcedUpdate() {
              // manual update echarts options
              this.needUpdate = true;
              this.$nextTick(() => { this.needUpdate = false; });
          }
      },
      render() {
          const attrs = vueDemi.isVue2
              ? { props: this.chartOpts }
              : this.chartOpts;
          return vueDemi.h(VeChart, attrs);
      }
  });

  core.use([charts.BarChart]);
  var Chart$8 = vueDemi.defineComponent({
      name: 'BarChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const BarChart = withInstall(Chart$8);

  core.use([charts.LineChart]);
  var Chart$7 = vueDemi.defineComponent({
      name: 'LineChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const LineChart = withInstall(Chart$7);

  core.use([charts.PieChart]);
  var Chart$6 = vueDemi.defineComponent({
      name: 'PieChart',
      extends: HocChart,
      setup() {
          import('echarts/charts')
              .then((module) => {
              console.log(module);
          });
      }
  });

  const PieChart = withInstall(Chart$6);

  core.use([charts.FunnelChart]);
  var Chart$5 = vueDemi.defineComponent({
      name: 'FunnelChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const FunnelChart = withInstall(Chart$5);

  core.use([charts.GaugeChart]);
  var Chart$4 = vueDemi.defineComponent({
      name: 'GaugeChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const GaugeChart = withInstall(Chart$4);

  core.use([charts.RadarChart]);
  var Chart$3 = vueDemi.defineComponent({
      name: 'RadarChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const RadarChart = withInstall(Chart$3);

  core.use([charts.ScatterChart]);
  var Chart$2 = vueDemi.defineComponent({
      name: 'ScatterChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const ScatterChart = withInstall(Chart$2);

  core.use([charts.TreeChart]);
  var Chart$1 = vueDemi.defineComponent({
      name: 'TreeChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const TreeChart = withInstall(Chart$1);

  core.use([charts.TreemapChart]);
  var Chart = vueDemi.defineComponent({
      name: 'TreemapChart',
      extends: HocChart,
      setup(props) {
          console.log(props);
      }
  });

  const TreemapChart = withInstall(Chart);

  const componentList = [
      VeChart,
      BarChart,
      LineChart,
      PieChart,
      FunnelChart,
      GaugeChart,
      RadarChart,
      ScatterChart,
      TreeChart,
      TreemapChart
  ];
  const install = (app) => {
      componentList.forEach(component => {
          app.component(component.name, component);
      });
  };

  exports.BarChart = BarChart;
  exports.FunnelChart = FunnelChart;
  exports.GaugeChart = GaugeChart;
  exports.LineChart = LineChart;
  exports.PieChart = PieChart;
  exports.RadarChart = RadarChart;
  exports.ScatterChart = ScatterChart;
  exports.TreeChart = TreeChart;
  exports.TreemapChart = TreemapChart;
  exports.VeChart = VeChart;
  exports.default = install;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
